#################################################################################
# Searching for G: A Critical Evaluation of SPM-LS dimensionality               #
#                                                                               #
# Appendix A                                                                    #
#                                                                               #
# Journal of Intelligence, 2019                                                 #                                       
#                                                                               #
# Authors:                                                                      #
#   1. Eduardo Garcia-Garzon                                                    #
#   1. Francisco J. Abad                                                        #  
#   2. Luis E. Garrido                                                          #
#                                                                               #  
# Affiliations:                                                                 #
#   1. Facultad de Psicologia, Universidad Autonoma de Madrid                   #    
#   2. Facultad de Psicologia, Pontificia Universidad Católica Madre y Maestra  #
#                                                                               #
#   Last updated: 20-06-2019                                                    #
#################################################################################


# -------------------------------------------------------------------------
# --------------- Loading packages & Data ---------------------------------
# -------------------------------------------------------------------------

path <- rstudioapi::getActiveDocumentContext()$path
Encoding(path) <- "UTF-8"
setwd(dirname(path))

if(!require(tidyverse))install.packages("tidyverse")
if(!require(DescTools))install.packages("DescTools")
if(!require(psych))install.packages("psych")
if(!require(EGAnet))install.packages("EGAnet")
if(!require(GPArotation))install.packages("GPArotation")
if(!require(qgraph))install.packages("qgraph")
if(!require(lavaan))install.packages("lavaan")
if(!require(semTools))install.packages("semTools")

## Loading a set of functions to be applied in the analyses:

source ("Auxiliary functions.R")

# Download data from original paper: 
# Myszkowski, N., & Storme, M. (2018). A snapshot of g. Binary and polytomous item-response theory investigations of the last series of the Standard Progressive Matrices (SPM-LS). 
# Intelligence, 68, 109-116.

poly.dat <- read.csv("https://data.mendeley.com/datasets/h3yhs5gy3w/1/files/11272d2a-aa63-43be-8413-145b9941d2d8/dataset.csv?dl=1", 
                     header=T)

# Apply data keys and estimate the polychoric correlation matrix

iq.keys <- c(7,6,8,2,1,5,1,6,3,2,4,5)
bin.dat <- score.multiple.choice(iq.keys, poly.dat, score = F)
bin.cor <- qgraph:::cor_auto(bin.dat)

ordered = c("SPM1" , "SPM2" , "SPM3" , "SPM4" , "SPM5" , "SPM6" , 
            "SPM7" , "SPM8" , "SPM9" , "SPM10" , "SPM11" , "SPM12")


# ---------------------------------------------------------------
# ---------------------------------------------------------------
# ------------------------ Appendix B. --------------------------
# ---------------------------------------------------------------
# ---------------------------------------------------------------

# ------------------------------------------------------
# --------------- Unidimensional CFA  ------------------
# ------------------------------------------------------

unidim <- "g =~ SPM1 + SPM2 + SPM3 + SPM4 + SPM5 + SPM6 + SPM7 + SPM8 + SPM9 + SPM10 + SPM11 + SPM12
           g ~~ 1*g"

fit.unim  <- sem(unidim, 
                 data = bin.dat,
                 ordered = ordered,
                 std.lv = T, 
                 estimator = "WLSMV")

fit.output <- summary(fit.unim, 
                      standardized = T, 
                      fit.measures = T)


## Standardized factor loadings
est <- standardizedSolution(fit.unim)
uni.est <- matrix(round(est$est.std[1:12], 3))

## Fit indexes
uni.fit <- fitMeasures(fit.unim, fit.measures = c("npar", "df", "chisq.scaled", 
                                                  "pvalue.scaled", "cfi.scaled", "tli.scaled",
                                                  "rmsea.scaled", "rmsea.ci.lower.scaled",
                                                  "rmsea.ci.upper.scaled", "srmr"))

#Cronbach's alpha 
round(reliability(fit.unim)[1,1],3)

#Omega hierarchical  
round(reliability(fit.unim)[3,1],3)


# -----------------------------------------------------
# --------------- Bi-idimensional EFA  ----------------
# -----------------------------------------------------

# Compute unrotated solution
efa.unrot <- efaUnrotate( data = bin.dat, 
                          nf = 2,  
                          ordered = ordered,
                          std.lv = T, 
                          estimator = "WLSMV")

bidim.est.efa <- standardizedSolution(efa.unrot)
bidim.est.efa <- matrix (ncol = 2, nrow =12, c(bidim.est.efa[1:12,4], bidim.est.efa[13:24,4]))

## Fit indexes
efa.unrot.fit <- fitMeasures(efa.unrot, fit.measures = c("npar", "df", "chisq.scaled", 
                                                         "pvalue.scaled", "cfi.scaled", "tli.scaled",
                                                         "rmsea.scaled", "rmsea.ci.lower.scaled",
                                                         "rmsea.ci.upper.scaled", "srmr"))

# Compute Target Matrix according to theoretical model.
target <- matrix (nrow = 12, ncol = 2, c(rep(NA, 6), rep(0,12), rep(NA, 6)))

# Compute targeted-rotated EFA solution.
bidim.efa <- targetQ(bidim.est.efa, 
                     Target = target)

# -----------------------------------------------------
# --------------- Bi-idimensional CFA  ----------------
# -----------------------------------------------------

bidim.cfa <- "g1 =~ SPM1 + SPM2 + SPM3 + SPM4 + SPM5 + SPM6 
              g2 =~ SPM7 + SPM8 + SPM9 + SPM10 + SPM11 + SPM12
              g1 ~~ 1*g1
              g2 ~~ 1*g2
              g1 ~~ NA*g2"

fit.bidim  <- sem(bidim.cfa, 
                  data = bin.dat,
                  ordered = ordered,
                  std.lv = T, 
                  estimator = "WLSMV")

fit.bidim.output <- summary(fit.bidim, 
                            standardized = T, 
                            fit.measures = T)

## Standardized factor loadings.
bidim.est <- standardizedSolution(fit.bidim)
bidim.est <- round(bidim.est$est.std[1:12], 3)
bidim.est <- matrix (ncol = 2, nrow =12, c(bidim.est[1:6], rep(0,12), bidim.est[7:12]))

## Residual correlation.
round(standardizedSolution(fit.bidim)$est.std[15],3)

## Fit indexes.
bidim.fit <- fitMeasures(fit.bidim, fit.measures = c("npar", "df", "chisq.scaled", 
                                                     "pvalue.scaled", "cfi.scaled", "tli.scaled",
                                                     "rmsea.scaled", "rmsea.ci.lower.scaled",
                                                     "rmsea.ci.upper.scaled", "srmr"))

#Cronbach's alpha for the first and second factor, respectively
round(reliability(fit.bidim)[1,1],3)
round(reliability(fit.bidim)[1,2],3)

#Omega hierarchical for the first and second factor, respectively
round(reliability(fit.bidim)[4,1],3)
round(reliability(fit.bidim)[4,2],3)


# -----------------------------------------------
# --------------- Bi-factor EFA  ----------------
# -----------------------------------------------

# Compute the bi-factor target matrix based on Garcia-Garzon, Garrido & Abad (2019) method (https://doi.org/10.1027/1614-2241/a000163).

# Compute unrotated solution.
efa.unrot <- efaUnrotate( data = bin.dat, 
                          nf = 2,  
                          ordered = ordered,
                          std.lv = T, 
                          estimator = "WLSMV")


est <- standardizedSolution(efa.unrot)[1:24,4]
est <- matrix(ncol=2, nrow=12, c(est))


## Fit indexes.
efa.unrot.fit <- fitMeasures(efa.unrot, fit.measures = c("npar", "df", "chisq.scaled", 
                                                         "pvalue.scaled", "cfi.scaled", "tli.scaled",
                                                         "rmsea.scaled", "rmsea.ci.lower.scaled",
                                                         "rmsea.ci.upper.scaled", "srmr"))

# Compute empirical partially specified target
Targ <- define_target(L = est)


# Rotate towards a BEFA model (oblique)
befa.fit <- targetQ(est, 
                    Target = Targ)

## Standardized factor loadings.
befa.est <- befa.fit$loadings

## Standardized factor correlation.
befa.phi <- befa.fit$Phi

# -----------------------------------------------
# --------------- Bi-factor CFA  ----------------
# -----------------------------------------------

bcfa <- "g  =~ SPM1 + SPM2 + SPM3 + SPM4 + SPM5 + SPM6 + SPM7 + SPM8 + SPM9 + SPM10 + SPM11 + SPM12
          f2 =~ SPM7 +  SPM8 + SPM9 + SPM10 + SPM11 + SPM12
          g ~~ 1*g
          f2 ~~ 1*f2
          g ~~ 0*f2"

fit.bcfa <- sem(bcfa,
                data = bin.dat,
                ordered = ordered,
                std.lv = T, 
                estimator = "WLSMV")


fit.bcfa.output <- summary(fit.bcfa, 
                           standardized = T, 
                           fit.measures = T)


## Standardized factor loadings.
bcfa.est <- standardizedSolution(fit.bcfa)
bcfa.est <- matrix(nrow = 12, ncol = 2, c(round(bcfa.est$est.std[1:12], 3), 
                                          rep (0,6), 
                                          round(bcfa.est$est.std[13:18], 3)))

## Residual correlation.
round(standardizedSolution(fit.bcfa)$est.std[19],3)

## Fit indexes.
bcfa.fit <- fitMeasures(fit.bcfa, fit.measures = c("npar", "df", "chisq.scaled", 
                                                   "pvalue.scaled", "cfi.scaled", "tli.scaled",
                                                   "rmsea.scaled", "rmsea.ci.lower.scaled",
                                                   "rmsea.ci.upper.scaled", "srmr"))

## Compute ECV
ecv(bcfa.est)

#Cronbach's alpha for the first and second factor, respectively
round(reliability(fit.bcfa)[1,1],3)
round(reliability(fit.bcfa)[1,2],3)

#Omega hierarchical for the first and second factor, respectively 
round(reliability(fit.bcfa)[4,1],3)
round(reliability(fit.bcfa)[4,2],3)
